<?php
declare (strict_types = 1);

namespace app;
use think\App;
use think\exception\HttpResponseException;
use think\exception\ValidateException;
use think\Validate;
use think\facade\View;
use think\facade\Config;
use app\common\library\Auth;
use app\common\library\Pub;
use think\facade\Request;

/**
* 控制器基础类
*/
abstract class BaseController
{
    /**
    * Request实例
    * @var \think\Request
    */
    protected $request;

    /**
    * 应用实例
    * @var \think\App
    */
    protected $app;

    /**
    * 是否批量验证
    * @var bool
    */
    protected $batchValidate = false;

    /**
    * 控制器中间件
    * @var array
    */
    protected $middleware = [];

    /**
    * 构造方法
    * @access public
    * @param  App  $app  应用对象
    */
    public function __construct(App $app) {
        $this->app = $app;
        $this->request = $this->app->request;
        if (!is_file(app()->getRootPath() . 'app/install/install.lock') && app('http')->getName() !== 'install') {
            return $this->redirectTo(url('install/index/index')->build());
        }
        // 控制器初始化
        $this->initialize();

    }
    public function redirectTo(...$args) {
        // 此处 throw new HttpResponseException 这个异常一定要写
        throw new HttpResponseException(redirect(...$args));
    }
    // 初始化
    protected function initialize() {}


    /**
    * 验证数据
    * @access protected
    * @param  array        $data     数据
    * @param  string|array $validate 验证器名或者验证规则数组
    * @param  array        $message  提示信息
    * @param  bool         $batch    是否批量验证
    * @return array|string|true
    * @throws ValidateException
    */
    protected function validate(array $data, $validate, array $message = [], bool $batch = false) {
        if (is_array($validate)) {
            $v = new Validate();
            $v->rule($validate);
        } else {
            if (strpos($validate, '.')) {
                // 支持场景
                [$validate,
                    $scene] = explode('.', $validate);
            }
            $class = false !== strpos($validate, '\\') ? $validate : $this->app->parseClass('validate', $validate);
            $v = new $class();
            if (!empty($scene)) {
                $v->scene($scene);
            }
        }

        $v->message($message);

        // 是否批量验证
        if ($batch || $this->batchValidate) {
            $v->batch(true);
        }

        return $v->failException(true)->check($data);
    }
    /**
    * 操作成功跳转的快捷方法
    * @access protected
    * @param  mixed     $msg 提示信息
    * @param  string    $url 跳转的URL地址
    * @param  mixed     $data 返回的数据
    * @param  integer   $wait 跳转等待时间
    * @param  array     $header 发送的Header信息
    * @return void
    */
    protected function success($msg = '', string $url = null, $data = '', int $wait = 1, array $header = []) {
        if (is_null($url) && isset($_SERVER["HTTP_REFERER"])) {
            $url = $this->request->isAjax() ? '' : 'javascript:history.back(-1);';
        } elseif ($url) {
            $url = (strpos($url, '://') || 0 === strpos($url, '/')) ? $url : app('route')->buildUrl($url);
        }

        $result = [
            'code' => 1,
            'msg' => $msg,
            'data' => $data,
            'url' => $url,
            'wait' => $wait,
        ];

        $type = $this->getResponseType();
        if ($type == 'html') {
            $response = view(config('app.dispatch_success_tmpl'), array_merge($result, ['cdn' => Config::get('website')['cdn']]));
        } else if ($type == 'json') {
            $response = json($result);
        }

        throw new HttpResponseException($response);
    }

    /**
    * 操作错误跳转的快捷方法
    * @access protected
    * @param  mixed     $msg 提示信息
    * @param  string    $url 跳转的URL地址
    * @param  mixed     $data 返回的数据
    * @param  integer   $wait 跳转等待时间
    * @param  array     $header 发送的Header信息
    * @return void
    */
    protected function error($msg = '', string $url = null, $data = '', int $wait = 3, array $header = []) {
        if (is_null($url)) {
            $url = $this->request->isAjax() ? '' : 'javascript:history.back(-1);';
        } elseif ($url) {
            $url = (strpos($url, '://') || 0 === strpos($url, '/')) ? $url : $this->app->route->buildUrl($url);
        }

        $result = [
            'code' => 0,
            'msg' => $msg,
            'data' => $data,
            'url' => $url,
            'wait' => $wait,
        ];

        $type = $this->getResponseType();
        if ($type == 'html') {
            $response = view(config('app.dispatch_success_tmpl'), array_merge($result, ['cdn' => Config::get('website')['cdn']]));
        } else if ($type == 'json') {
            $response = json($result);
        }

        throw new HttpResponseException($response);
    }
    /**
    * 获取当前的response 输出类型
    * @access protected
    * @return string
    */
    protected function getResponseType() {
        return $this->request->isJson() || $this->request->isAjax() ? 'json' : 'html';
    }
    /**
    * 设置模板变量
    * @access protected
    * @param string|array $name
    * @param array $value
    * @return View
    */
    protected function assign($name, $value = []) {
        return View::assign($name, $value);
    }
}