<?php
declare (strict_types = 1);

namespace app\install\controller;
use app\common\controller\Json;
use app\install\validate\ConfigSubmit;
use think\exception\ValidateException;
use think\facade\Db;
use think\Request;
use think\facade\Config;
use app\BaseController;
class Index extends BaseController
{
    public function initialize() {
        // 重复安装检测
        if (is_file(app()->getRootPath() . 'app/install/install.lock')) {
            exitContent(Json::error('当前网站已安装, 如需重复安装请删除/app/install/install.lock')->res());
        }

        // 当前方法
        $method = request()->action(true);

        // 环境检测
        if (!in_array($method, ['index', 'getserverinfo'])) {
            !$this->getInstalldetect(true) && exitContent(Json::error('环境检测不通过')->res());
        }

        // 配置检测
        if (in_array($method, ['startinstall'])) {
            !session('install_params') && exitContent(Json::error('请先进行配置后再开始安装')->res());
        }
        $web = Config::get('website');
        $this->assign('web',$web);
    }
    public static function getInstalldetect($bool = false) {
        $data = [];

        // PHP版本校验
        $data[] = ['name' => 'PHP版本', 'current' => PHP_VERSION, 'demand' => '7.1.0', 'status' => version_compare(PHP_VERSION, '7.1.0', '>')];
        $data[] = ['name' => '类', 'current' => '--', 'demand' => 'pdo', 'status' => class_exists('PDO')];
        $data[] = ['name' => '类', 'current' => '--', 'demand' => 'ZipArchive', 'status' => class_exists('ZipArchive')];
        $data[] = ['name' => '函数', 'current' => '--', 'demand' => 'curl', 'status' => function_exists('curl_exec')];

        // 返回检测结果
        if ($bool) {
            foreach ($data as $item) {
                if (!$item['status']) {
                    return false;
                }
            }
        }

        return $data;
    }
    /**
     * 程序安装首页
     * @return \think\Response
     */
    public function index() {
        return view();
    }

    /**
     * 获取服务器信息
     * @access public
     * @return mixed
     */
    public function getServerInfo() {
        return Json::success()->data($this->getInstalldetect())->res();
    }

    /**
     * 安装配置提交
     * @access public
     * @param Request $request
     * @return mixed
     */
    public function installConfigSubmit(Request $request) {
        // 获取表单信息
        $data = $request->param(['adminurl','db_hostname', 'db_hostport', 'db_username', 'db_password', 'db_database', 'username', 'password', 'name'], null);

        try {
            // 表单验证
            validate(ConfigSubmit::class)->check($data);
            // 数据库连接测试
            $this->db($data['db_hostname'], $data['db_hostport'], $data['db_username'], $data['db_password'], $data['db_database']);
            // 写配置文件
            $this->writeEnvFile($data);
            // 记录管理员信息
            session('install_params', ['adminurl'=>$data['adminurl'],'username' => $data['username'], 'password' => $data['password'], 'name' => $data['name']]);
            return Json::success()->res();
        } catch (ValidateException $e) {
            return Json::error($e->getMessage())->res();
        } catch (UserError $e) {
            return Json::error($e->getMessage())->res();
        }
    }

    /**
     * 开始安装
     * @access public
     * @return mixed
     */
    public function startInstall() {
        try {
            // 导入数据库
            list($status, $info) = $this->importSql(session('install_params'));
            if (!$status) {
                return Json::error('导入数据库失败, Error: ' . $info)->res();
            }

            // 写入安装锁
            if (!file_put_contents(app()->getRootPath() . 'app/install/install.lock', 'success')) {
                return Json::error('写入安装锁失败, 请检查权限 ')->res();
            }
            return Json::success()->res();
        } catch (\Exception $e) {} catch (\ParseError $e) {}

        $msg = '安装时出现错误';
        $msg .= ', Error: ' . $e->getMessage() . '<br>';
        $msg .= $e->getFile() . ':' . $e->getLine();

        return Json::error($msg)->res();
    }

    /**
     * 新建数据库实例
     * @access protected
     * @param $host
     * @param $port
     * @param $user
     * @param $pass
     * @param $dbName
     * @return void
     */
    protected function db($host, $port, $user, $pass, $dbName) {
        try {
            $dsn = 'mysql:host=' . $host . ';dbname=' . $dbName . ';port=' . $port . ';charset=utf8';
            $link = new \PDO($dsn, $user, $pass);
            $link = null;
        } catch (\PDOException $e) {
            exitContent(Json::error('数据库连接失败, Error: ' . $e->getMessage())->res());
        }
    }

    /**
     * 写ENV配置文件
     * @access protected
     * @param $config
     * @return void
     * @throws UserError
     */
    protected function writeEnvFile($config) {
        $content = <<<TEXT
APP_DEBUG = false

[APP]
DEFAULT_TIMEZONE = Asia/Shanghai

[DATABASE]
TYPE = mysql
HOSTNAME = {$config['db_hostname']}
DATABASE = {$config['db_database']}
USERNAME = {$config['db_username']}
PASSWORD = {$config['db_password']}
HOSTPORT = {$config['db_hostport']}
CHARSET = utf8
DEBUG = false

[LANG]
default_lang = zh-cn
TEXT;
        if (!file_put_contents(app()->getRootPath() . '.env', $content)) {
            exitContent(Json::error('写入配置文件失败, 请检查权限')->res());
        }
    }

    /**
     * 导入SQL
     * @access protected
     * @param array $params 参数列表
     * @return array
     */
    protected function importSql($params) {
        // 获取SQL列表
      $list = require __DIR__ . '/../' . 'sql.php';

        try {
            foreach ($list as $sql) {
                Db::query($sql);
            }


            // 插入管理员信息
            Db::table('admin')->where(['id' => '1'])->update(['adminuser' => $params['username'], 'adminpwd' => password_hash($params['password'], PASSWORD_DEFAULT), 'adminqq' => $params['name']]);
            if(!rename(app()->getRootPath() . 'public/admin.php',app()->getRootPath() . 'public/'.$params['adminurl'])){
                return [false,'写入后台入口文件失败, 请检查权限']; 
            }
            return [true, null];
        } catch (\Exception $e) {
            return [false, $e->getMessage()];
        }
    }
}